from django.contrib import admin
from feincms.admin.item_editor import ItemEditor
from feincms.extensions import ExtensionModelAdmin
from orderable.admin import OrderableTabularInline

from . import models


class PageAdmin(ItemEditor, ExtensionModelAdmin):
    fieldsets = [
        (None, {
            'fields': ['name', 'slug'],
        }),
        # <-- extensions appear here, see fieldset_insertion_index.
    ]
    fieldset_insertion_index = 1
    filter_horizontal = []
    list_display = ['name', 'slug']
    list_filter = []
    prepopulated_fields = {'slug': ['name']}
    search_fields = ['name', 'slug']


class GroupItemInline(OrderableTabularInline):
    model = models.GroupItem


class GroupAdmin(admin.ModelAdmin):
    inlines = [GroupItemInline]


admin.site.register(models.Page, PageAdmin)
admin.site.register(models.Group, GroupAdmin)



from rest_framework import serializers
from rest_framework.reverse import reverse

from pages import fields, mixins, models
from pages.utils import build_url


class PageSerializer(serializers.HyperlinkedModelSerializer):
    name = serializers.CharField()
    regions = serializers.SerializerMethodField('rendered_regions')

    class Meta:
        fields = ('id', 'url', 'name', 'slug', 'regions')
        model = models.Page
        view_name = 'pages:page-detail'
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
        }

    def rendered_regions(self, obj):
        return obj.rendered_regions(self.context['request'])


class JsonPageSerializer(PageSerializer):
    def rendered_regions(self, obj):
        """Render regions as a json-serializable dictionary."""
        return obj.render_json(self.context.get('request'))


class GroupSerializer(mixins.LinksMixin, serializers.HyperlinkedModelSerializer):
    url = fields.AbsoluteURLIdentityField()
    pages = serializers.SerializerMethodField('get_pages_link')

    links_fields = ['pages']

    class Meta:
        model = models.Group

    def get_pages_link(self, obj):
        return build_url(
            reverse('pages:page-list', request=self.context.get('request')),
            {'group': obj.slug},
        )



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Page'
        db.create_table('pages_page', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('pages', ['Page'])

        # Adding model 'Group'
        db.create_table('pages_group', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('pages', ['Group'])

        # Adding model 'GroupItem'
        db.create_table('pages_groupitem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(db_index=True, blank=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Group'])),
        ))
        db.send_create_signal('pages', ['GroupItem'])


    def backwards(self, orm):
        # Deleting model 'Page'
        db.delete_table('pages_page')

        # Deleting model 'Group'
        db.delete_table('pages_group')

        # Deleting model 'GroupItem'
        db.delete_table('pages_groupitem')


    models = {
        'pages.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'})
        },
        'pages.groupitem': {
            'Meta': {'object_name': 'GroupItem', 'ordering': "['sort_order']"},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['pages.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['pages.Page']"}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'blank': 'True'})
        },
        'pages.page': {
            'Meta': {'object_name': 'Page'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['pages']


from rest_framework import generics, viewsets
from rest_framework.permissions import IsAuthenticatedOrReadOnly

from . import models, serializers


class PageViewSet(viewsets.ReadOnlyModelViewSet):
    lookup_field = 'slug'
    queryset = models.Page.objects.all()
    permission_classes = [IsAuthenticatedOrReadOnly]

    def get_serializer_class(self):
        """Get the appropriate serializer using the regions_format."""
        regions_format = self.request.QUERY_PARAMS.get('regions_format')
        if regions_format == 'json':
            return serializers.JsonPageSerializer
        return serializers.PageSerializer

    def get_queryset(self):
        queryset = super(PageViewSet, self).get_queryset()
        group_slug = self.request.QUERY_PARAMS.get('group')

        if group_slug is None:
            return queryset

        return queryset.filter(groupitem__group__slug=group_slug)


class GroupView(generics.RetrieveAPIView):
    lookup_field = 'slug'
    queryset = models.Group.objects.all()
    serializer_class = serializers.GroupSerializer



from django.core.urlresolvers import resolve, reverse
from django.test import TestCase

from .. import views


class TestURLs(TestCase):
    """Ensure the pages urls work."""
    def test_page_detail_url(self):
        url = reverse('pages:page-detail', kwargs={'slug': 'slug'})
        view_name = resolve(url).func.__name__
        self.assertEqual(view_name, views.PageViewSet.__name__)

    def test_page_list_url(self):
        url = reverse('pages:page-list')
        view_name = resolve(url).func.__name__
        self.assertEqual(view_name, views.PageViewSet.__name__)

    def test_group_url(self):
        url = reverse('pages:group-detail', kwargs={'slug': 'slug'})
        view_name = resolve(url).func.__name__
        self.assertEqual(view_name, views.GroupView.__name__)



from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from feincms_extensions.content_types import JsonRichTextContent
from user_management.models.mixins import ActiveUserMixin, BasicUserFieldsMixin

from pages.models import Page


class User(ActiveUserMixin, BasicUserFieldsMixin, PermissionsMixin, AbstractBaseUser):
    pass


Page.register_regions(
    ('abstract', 'Abstract/Summary'),
    ('body', 'Main Article'),
)
Page.create_content_type(JsonRichTextContent)
Page.register_extensions(
    'feincms_extensions.render_regions',
    'feincms_extensions.render_json',
)



import factory

from .. import models


class PageFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Page

    name = factory.Sequence(lambda n: 'Page {}'.format(n))
    slug = factory.Sequence(lambda n: 'page-{}'.format(n))


class GroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Group

    slug = factory.Sequence(lambda n: 'group-{}'.format(n))


class GroupItemFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.GroupItem

    page = factory.SubFactory(PageFactory)
    group = factory.SubFactory(GroupFactory)
    sort_order = factory.Sequence(lambda n: n)



from django.test import TestCase
from incuna_test_utils.compat import wipe_id_fields_on_django_lt_17
from mock import MagicMock, patch

from . import factories
from .. import models


class TestPageModel(TestCase):
    def test_fields(self):
        expected = {
            'id',
            'name',
            'slug',
            'groupitem',
            'jsonrichtextcontent_set',
        }

        fields = models.Page._meta.get_all_field_names()

        # assertItemsEqual has been renamed assertCountEqual in python version 3
        try:
            # python 3
            self.assertCountEqual(fields, expected)
        except AttributeError:
            # python 2.7
            self.assertItemsEqual(fields, expected)

    def test_slug_unique(self):
        slug_field = models.Page._meta.get_field_by_name('slug')[0]
        self.assertTrue(slug_field.unique)

    def test_str(self):
        page = factories.PageFactory.build()
        self.assertEqual(str(page), page.name)


class TestGroupModel(TestCase):
    def test_fields(self):
        expected = {
            'id',
            'slug',
            'groupitem',
        }

        fields = models.Group._meta.get_all_field_names()

        # assertItemsEqual has been renamed assertCountEqual in python version 3
        try:
            # python 3
            self.assertCountEqual(fields, expected)
        except AttributeError:
            # python 2.7
            self.assertItemsEqual(fields, expected)

    def test_str(self):
        group = factories.GroupFactory.build()
        self.assertEqual(str(group), group.slug)

    def test_get_absolute_url(self):
        group = factories.GroupFactory.create()
        request = MagicMock()

        with patch('pages.models.reverse') as patched_reverse:
            group.get_absolute_url(request)

        patched_reverse.assert_called_once_with(
            'pages:group-detail',
            kwargs={'slug': group.slug},
            request=request,
        )


class TestGroupItem(TestCase):
    def test_fields(self):
        expected = wipe_id_fields_on_django_lt_17({
            'id',
            'page',
            'page_id',
            'group',
            'group_id',
            'sort_order',
        })

        fields = models.GroupItem._meta.get_all_field_names()

        # assertItemsEqual has been renamed assertCountEqual in python version 3
        try:
            # python 3
            self.assertCountEqual(fields, expected)
        except AttributeError:
            # python 2.7
            self.assertItemsEqual(fields, expected)



from mock import patch
from rest_framework.reverse import reverse
from user_management.models.tests.utils import APIRequestTestCase

from pages import views
from pages.tests import factories
from pages.tests.utils import TEST_SERVER
from pages.utils import build_url


class TestPageViewSet(APIRequestTestCase):
    view_class = views.PageViewSet

    def get_expected(self, page):
        return {
            'id': page.pk,
            'url': TEST_SERVER + page.get_absolute_url(),
            'name': page.name,
            'slug': page.slug,
            'regions': {
                'abstract': '',
                'body': 'Wow!',
            },
        }

    def test_get_detail(self):
        """Test GET Page detail."""
        page = factories.PageFactory.create()
        page.jsonrichtextcontent_set.create(region='body', text='Wow!')

        request = self.create_request()
        view = self.view_class.as_view({'get': 'retrieve'})
        response = view(request, slug=page.slug)
        self.assertEqual(response.status_code, 200)

        expected = self.get_expected(page)
        self.assertEqual(response.data, expected)

    def test_get_list(self):
        """Test GET Page list."""
        page = factories.PageFactory.create()
        page.jsonrichtextcontent_set.create(region='body', text='Wow!')

        request = self.create_request()
        response = self.view_class.as_view({'get': 'list'})(request)
        self.assertEqual(response.status_code, 200)

        expected = [self.get_expected(page)]
        self.assertEqual(response.data, expected)

    def test_filtered_list(self):
        """Test filtering by Group."""
        page, other_page = factories.PageFactory.create_batch(2)
        group = factories.GroupFactory.create()
        factories.GroupItemFactory.create(page=page, group=group)

        request = self.create_request()
        request.QUERY_PARAMS = {'group': group.slug}
        view = self.view_class()
        view.request = request
        queryset = view.get_queryset()

        self.assertIn(page, queryset)
        self.assertNotIn(other_page, queryset)

    def test_get_detail_anonymous(self):
        """Test GET Page detail unauthenticated."""
        page = factories.PageFactory.create()
        page.jsonrichtextcontent_set.create(region='body', text='Wow!')

        request = self.create_request(auth=False)
        view = self.view_class.as_view({'get': 'retrieve'})
        response = view(request, slug=page.slug)
        self.assertEqual(response.status_code, 200)

        expected = self.get_expected(page)
        self.assertEqual(response.data, expected)

    def test_get_detail_json_regions(self):
        """A Page's regions can be retrieved as json."""
        page = factories.PageFactory.create()
        text = 'Wow!'
        body_text = page.jsonrichtextcontent_set.create(region='body', text=text)

        request = self.create_request(data={'regions_format': 'json'})
        view = self.view_class.as_view({'get': 'retrieve'})
        response = view(request, slug=page.slug)
        self.assertEqual(response.status_code, 200)

        expected = self.get_expected(page)
        expected['regions'] = {
            'abstract': [],
            'body': [{'html': text, 'content_type': 'rich-text', 'id': body_text.id}],
        }
        self.assertEqual(response.data, expected)


class TestGroupView(APIRequestTestCase):
    view_class = views.GroupView

    url_path = 'pages.models.Group.get_absolute_url'
    mocked_url = '/mocked_url'

    def get_expected(self, group, request):
        slug = group.slug
        return {
            'url': self.mocked_url,
            'slug': slug,
            'links': {
                'pages': build_url(
                    reverse('pages:page-list', request=request),
                    {'group': slug},
                ),
            },
        }

    @patch(url_path)
    def test_get(self, group_url):
        group_url.return_value = self.mocked_url
        group = factories.GroupFactory.create()

        request = self.create_request()
        view = self.view_class.as_view()
        response = view(request, slug=group.slug)
        self.assertEqual(response.status_code, 200)

        expected = self.get_expected(group, request)
        self.assertEqual(response.data, expected)



import mock
from django.test import TestCase

from ..fields import AbsoluteURLIdentityField


class AbsoluteURLIdentityFieldTest(TestCase):
    def test_get_url(self):
        field = AbsoluteURLIdentityField()
        obj = mock.MagicMock()
        view_name = mock.MagicMock()
        request = mock.MagicMock()
        format = mock.MagicMock()

        with mock.patch.object(obj, 'get_absolute_url') as get_absolute_url:
            field.get_url(obj, view_name, request, format)
        get_absolute_url.assert_called_once_with(request)



from django.conf.urls import include, url
from rest_framework.routers import SimpleRouter

from pages import views


router = SimpleRouter(trailing_slash=False)
router.register(r'pages', views.PageViewSet, base_name='page')

urlpatterns = [
    url(r'', include(router.urls)),
]



from django.conf.urls import url

from pages import views


urlpatterns = [
    url(
        r'^group/(?P<slug>[a-z0-9-]+)/?',
        views.GroupView.as_view(),
        name='group-detail',
    ),
]



from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms.models import Base
from orderable.models import Orderable
from rest_framework.reverse import reverse


class Page(Base):
    name = models.CharField(_('name'), max_length=255)
    slug = models.SlugField(_('slug'), max_length=255, unique=True)

    def get_absolute_url(self, request=None):
        """
        Returns the absolute url.

        If a request is passed the url will include the domain and port.
        """
        return reverse(
            'pages:page-detail', kwargs={'slug': self.slug}, request=request)

    def __str__(self):
        return self.name


class Group(models.Model):
    slug = models.SlugField(_('slug'), max_length=255, unique=True)

    def __str__(self):
        return self.slug

    def get_absolute_url(self, request=None):
        return reverse(
            'pages:group-detail', kwargs={'slug': self.slug}, request=request)


class GroupItem(Orderable):
    page = models.ForeignKey(Page)
    group = models.ForeignKey(Group)



class LinksMixin(object):
    links_fields = []

    def to_representation(self, obj):
        fields = super(LinksMixin, self).to_representation(obj)
        fields['links'] = {k: fields.pop(k) for k in self.links_fields if k in fields}
        return fields



from collections import OrderedDict
from unittest import TestCase

from pages.utils import build_url


class TestUtils(TestCase):
    def test_build_url(self):
        url = 'example.com/test/'
        expected = 'example.com/test/?foo=bar&spam=eggs'

        get_params = OrderedDict((('foo', 'bar'), ('spam', 'eggs')))
        self.assertEqual(expected, build_url(url, get=get_params))



from rest_framework.reverse import reverse
from user_management.models.tests.utils import APIRequestTestCase

from pages import serializers
from pages.tests import factories
from pages.utils import build_url


class PageSerializerTest(APIRequestTestCase):
    def setUp(self):
        self.request = self.create_request()
        self.context = {'request': self.request}

    def expected_data(self, page):
        expected = {
            'id': page.pk,
            'url': page.get_absolute_url(self.request),
            'name': page.name,
            'slug': page.slug,
            'regions': page.rendered_regions(self.request),
        }
        return expected

    def test_serialize(self):
        page = factories.PageFactory.create()
        serializer = serializers.PageSerializer(page, context=self.context)
        self.assertEqual(serializer.data, self.expected_data(page))

    def test_serialize_to_json(self):
        page = factories.PageFactory.create()
        serializer = serializers.JsonPageSerializer(page, context=self.context)

        expected_data = self.expected_data(page)
        expected_data['regions'] = {'abstract': [], 'body': []}
        self.assertEqual(serializer.data, expected_data)


class GroupTest(APIRequestTestCase):
    def setUp(self):
        self.request = self.create_request()
        self.context = {'request': self.request}

    def expected_data(self, group):
        slug = group.slug
        return {
            'url': group.get_absolute_url(self.request),
            'slug': slug,
            'links': {
                'pages': build_url(
                    reverse('pages:page-list', request=self.request),
                    {'group': slug},
                ),
            },
        }

    def test_serialize(self):
        group = factories.GroupFactory.create()

        serializer = serializers.GroupSerializer(group, context=self.context)
        self.assertEqual(serializer.data, self.expected_data(group))



from setuptools import find_packages, setup


version = '1.1.0'


install_requires = (
    'djangorestframework>=3.0.5,<3.2',
    'FeinCMS>=1.9,<1.11',
    'django-orderable>=2.0.1,<4',
    'feincms-extensions>=0.1.0,<1',
)


setup(
    name='feincms-pages-api',
    version=version,
    author='Incuna Ltd',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/feincms-pages-api/',
    packages=find_packages(),
    install_requires=install_requires,
    include_package_data=True,
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
)



from rest_framework import serializers


class AbsoluteURLMixin(object):
    """Use get_absolute_url to get the field's api endpoint."""

    def __init__(self, *args, **kwargs):
        # HyperlinkedFields expect a view_name to be passed, so pass ''.
        kwargs.setdefault('view_name', '')
        super(AbsoluteURLMixin, self).__init__(*args, **kwargs)

    def get_url(self, obj, view_name, request, format):
        return obj.get_absolute_url(request)


class AbsoluteURLIdentityField(AbsoluteURLMixin, serializers.HyperlinkedIdentityField):
    pass



#! /usr/bin/env python
"""From http://stackoverflow.com/a/12260597/400691"""
import sys

import dj_database_url
import django
from colour_runner.django_runner import ColourRunnerMixin
from django.conf import settings


settings.configure(
    DATABASES={
        'default': dj_database_url.config(default='postgres://localhost/pages'),
    },
    DEFAULT_FILE_STORAGE='inmemorystorage.InMemoryStorage',
    INSTALLED_APPS=(
        'pages',
        'pages.tests',  # This adds the test User model.

        # 3rd party app
        'feincms',
        'user_management.api',

        # Put contenttypes before auth to work around test issue.
        # See: https://code.djangoproject.com/ticket/10827#comment:12
        'django.contrib.contenttypes',
        'django.contrib.auth',
        'django.contrib.admin',
    ),
    MIDDLEWARE_CLASSES=(),
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
    SITE_ID=1,
    AUTH_USER_MODEL='tests.User',
    ROOT_URLCONF='pages.tests.urls',
)


if django.VERSION >= (1, 7):
    django.setup()


# DiscoverRunner requires `django.setup()` to have been called
from django.test.runner import DiscoverRunner  # noqa


class TestRunner(ColourRunnerMixin, DiscoverRunner):
    """Enable colorised output."""

test_runner = TestRunner(verbosity=1)
failures = test_runner.run_tests(['pages'])
if failures:
    sys.exit(1)
